@@ -638,7 +638,7 @@ do_argfile(exarg_T *eap, int argn)
     char_u	*p;
     int		old_arg_idx = curwin->w_arg_idx;
 
-    if (ERROR_IF_POPUP_WINDOW)
+    if (ERROR_IF_ANY_POPUP_WINDOW)
 	return;
     if (argn < 0 || argn >= ARGCOUNT)
     {
@@ -1046,7 +1046,7 @@ do_arg_all(
 	    // Move the already present window to below the current window
 	    if (curwin->w_arg_idx != i)
 	    {
-		for (wpnext = firstwin; wpnext != NULL; wpnext = wpnext->w_next)
+		FOR_ALL_WINDOWS(wpnext)
 		{
 		    if (wpnext->w_arg_idx == i)
 		    {
