@@ -389,7 +389,7 @@ calc_hist_idx(int histype, int num)
 		i += hislen;
 		wrapped = TRUE;
 	    }
-	if (hist[i].hisnum == num && hist[i].hisstr != NULL)
+	if (i >= 0 && hist[i].hisnum == num && hist[i].hisstr != NULL)
 	    return i;
     }
     else if (-num <= hislen)
