@@ -634,19 +634,6 @@ py3__Py_XDECREF(PyObject *op)
 #  define Py_XDECREF(op) py3__Py_XDECREF(_PyObject_CAST(op))
 # endif
 
-/*
- * Free python.dll
- */
-    static void
-end_dynamic_python3(void)
-{
-    if (hinstPy3 != 0)
-    {
-	close_dll(hinstPy3);
-	hinstPy3 = 0;
-    }
-}
-
 /*
  * Load library and get all pointers.
  * Parameter 'libname' provides name of DLL.
@@ -873,10 +860,6 @@ python3_end(void)
 	Py_Finalize();
     }
 
-#ifdef DYNAMIC_PYTHON3
-    end_dynamic_python3();
-#endif
-
     --recurse;
 }
 
